@@ -8,7 +8,7 @@ package Net::SMTPS;
 
 use vars qw ( $VERSION @ISA );
 
-$VERSION = '0.06';
+$VERSION = '0.10';
 
 use strict;
 use base qw ( Net::SMTP );
@@ -40,7 +40,9 @@ sub new {
       $host = delete $arg{Host};
   }
   my $ssl = delete $arg{doSSL};
-  $ssl = 'ssl' if delete $arg{SSL};
+  if ($ssl =~ /ssl/i) {
+      $arg{Port} ||= 465;
+  }
 
   my $hosts = defined $host ? $host : $NetConfig{smtp_hosts};
   my $obj;
@@ -77,23 +79,11 @@ sub new {
 
   $obj->debug(exists $arg{Debug} ? $arg{Debug} : undef);
 
-# common in SSL
-  my %ssl_args;
-  if ($ssl) {
-    eval {
-      require IO::Socket::SSL;
-    } or do {
-      $obj->set_status(500, ["Need working IO::Socket::SSL"]);
-      $obj->close;
-      return undef;
-    };
-    %ssl_args = map { +"$_" => $arg{$_} } grep {/^SSL/} keys %arg;
-    $IO::Socket::SSL::DEBUG = (exists $arg{Debug} ? $arg{Debug} : undef); 
-  }
+  ${*$obj}{'net_smtp_arg'} = \%arg;
 
 # OverSSL
   if (defined($ssl) && $ssl =~ /ssl/i) {
-    $obj->ssl_start(\%ssl_args)
+    $obj->ssl_start()
       or do {
 	$obj->set_status(500, ["Cannot start SSL"]);
 	$obj->close;
@@ -118,63 +108,94 @@ sub new {
   }
 
 # STARTTLS
-  if (defined($ssl) && $ssl =~ /starttls/i && $obj->supports('STARTTLS') ) {
-    (($obj->command('STARTTLS')->response() == CMD_OK)
-     and $obj->ssl_start(\%ssl_args)
-     and $obj->hello($arg{Hello} || ""))
-       or do {
-	 $obj->set_status(500, ["Cannot start SSL session"]);
-	 $obj->close();
-	 return undef;
-       };
+  if (defined($ssl) && $ssl =~ /starttls/i && defined($obj->supports('STARTTLS')) ) {
+      #123006 $obj->supports('STARTTLS') returns '' issue.
+      unless ($obj->starttls()) {
+	  return undef;
+      }
+      $obj->hello($arg{Hello} || "");
   }
 
   $obj;
 }
 
 sub ssl_start {
-    my ($self, $args) = @_;
+    my $self = shift;
     my $type = ref($self);
+    my %arg = %{ ${*$self}{'net_smtp_arg'} };
+    my %ssl_args = map { +"$_" => $arg{$_} } grep {/^SSL/} keys %arg;
 
+    eval {
+      require IO::Socket::SSL;
+    } or do {
+      $self->set_status(500, ["Need working IO::Socket::SSL"]);
+      $self->close;
+      return undef;
+    };
+
+    my $ssl_debug = (exists $arg{Debug} ? $arg{Debug} : undef);
+    $ssl_debug = (exists $arg{Debug_SSL} ? $arg{Debug_SSL} : $ssl_debug);
+    
+    local $IO::Socket::SSL::DEBUG = $ssl_debug; 
+     
     (unshift @ISA, 'IO::Socket::SSL'
-     and IO::Socket::SSL->start_SSL($self, %$args)
+     and IO::Socket::SSL->start_SSL($self, %ssl_args, @_)
      and $self->isa('IO::Socket::SSL')
      and bless $self, $type     # re-bless 'cause IO::Socket::SSL blesses himself.
     ) or return undef;
 }
 
+sub starttls {
+    my $self = shift;
+    (
+     $self->_STARTTLS()
+     and $self->ssl_start(@_)
+    ) or do {
+	$self->set_status(500, ["Cannot start SSL session"]);
+	$self->close();
+	return undef;
+    };
+}
+
+
 # Override to specify a certain auth mechanism.
 sub auth {
-  my ($self, $username, $password, $mech) = @_;
+    my ($self, $username, $password, $mech) = @_;
 
-  if ($mech) {
-      $self->debug_print(1, "my favorite: ". $mech . "\n") if $self->debug;
+    if ($mech) {
+	$self->debug_print(1, "AUTH-my favorite: ". $mech . "\n") if $self->debug;
 
-      my @cl_mech = split /\s+/, $mech;
-      my @matched = ();
-      if (exists ${*$self}{'net_smtp_esmtp'}->{'AUTH'}) {
-	  my $sv = ${*$self}{'net_smtp_esmtp'}->{'AUTH'};
-	  foreach my $i (@cl_mech) {
-	      if (index($sv, $i) >= 0 && grep(/$i/i, @matched) == () ) {
-		  push @matched, uc($i);
-	      }
-	  }
-      }
-      if (@matched) {
-	  ## override AUTH mech as specified.
-	  ## if multiple mechs are specified, priority is still up to Authen::SASL module.
-	  ${*$self}{'net_smtp_esmtp'}->{'AUTH'} = join " ", @matched;
-      }
-  }
-  $self->SUPER::auth($username, $password);
+	my @cl_mech = split /\s+/, $mech;
+	my @matched = ();
+	if (exists ${*$self}{'net_smtp_esmtp'}->{'AUTH'}) {
+	    my $sv = ${*$self}{'net_smtp_esmtp'}->{'AUTH'};
+	    $self->debug_print(1, "AUTH-server offerred: ". $sv . "\n") if $self->debug;
+
+	    foreach my $i (@cl_mech) {
+		if (index($sv, $i) >= 0 && !grep(/$i/i, @matched)) {
+		    push @matched, uc($i);
+		}
+	    }
+	}
+	if (@matched) {
+	    ## override AUTH mech as specified.
+	    ## if multiple mechs are specified, priority is still up to Authen::SASL module.
+	    ${*$self}{'net_smtp_esmtp'}->{'AUTH'} = join " ", @matched;
+	    $self->debug_print(1, "AUTH-negotiated: ". ${*$self}{'net_smtp_esmtp'}->{'AUTH'} . "\n") if $self->debug;
+	}
+    }
+    $self->SUPER::auth($username, $password);
 }
 
+
 # Fix #121006 no timeout issue.
 sub getline {
-  my $self = shift;
-  $self->Net::Cmd::getline(@_);
+    my $self = shift;
+    $self->Net::Cmd::getline(@_);
 }
 
+sub _STARTTLS { shift->command("STARTTLS")->response() == CMD_OK }
+
 1;
 
 __END__
@@ -194,19 +215,32 @@ Net::SMTPS - SSL/STARTTLS support for Net::SMTP
 =head1 DESCRIPTION
 
 This module implements a wrapper for Net::SMTP, enabling over-SSL/STARTTLS support.
-This module inherits all the methods from Net::SMTP. You may use all the friendly
-options that came bundled with Net::SMTP.
+This module inherits most of all the methods from Net::SMTP(2.X). You may use all
+the friendly options that came bundled with Net::SMTP.
 You can control the SSL usage with the options of new() constructor method.
 'doSSL' option is the switch, and, If you would like to control detailed SSL settings,
 you can set SSL_* options that are brought from IO::Socket::SSL. Please see the
 document of IO::Socket::SSL about these options detail.
 
-Just one method difference from the Net::SMTP, you may select SMTP AUTH mechanism
+Just one method difference from the Net::SMTP, you can select SMTP AUTH mechanism
 as the third option of auth() method.
 
 As of Version 3.10 of Net::SMTP(libnet) includes SSL/STARTTLS capabilities, so
 this wrapper module's significance disappareing.
 
+=head1 CONSTRUCTOR
+
+=over 4
+
+=item new ( [ HOST ] [, OPTIONS ] )
+
+A few options added to Net::SMTP(2.X).
+
+B<doSSL> { C<ssl> | C<starttls> | undef } - to specify SSL connection type.
+C<ssl> makes connection wrapped with SSL, C<starttls> uses SMTP command C<STARTTLS>.
+
+=back
+
 =head1 METHODS
 
 Most of all methods of Net::SMTP are inherited as is, except auth().
@@ -220,8 +254,17 @@ Attempt SASL authentication through Authen::SASL module. AUTHMETHOD is your requ
 method of authentication, like 'CRAM-MD5', 'LOGIN', ... etc. If your selection does
 not match the server-offerred AUTH mechanism, authentication negotiation may fail.
 
+=item starttls ( SSLARGS )
+
+Upgrade existing plain connection to SSL.
+
 =back
 
+=head1 BUGS
+
+Constructor option 'Debug => (N)' (for Net::Cmd) also sets $IO::Socket::SSL::DEBUG when SSL is enabled. You can set 'Debug_SSL => {0-3}' separately.
+
+
 =head1 SEE ALSO
 
 L<Net::SMTP>,
@@ -234,7 +277,7 @@ Tomo.M <tomo at cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2017 Tomo.M All rights reserved.
+Copyright (c) 2020 Tomo.M All rights reserved.
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
